# Record Send Radio Script (RSRS)
# Version 0.1
# 18-03-2024

import subprocess
from datetime import datetime
import os
import sys
import json
from ftplib import FTP, error_perm
from webdav3.client import Client

# Ustalanie bieżącego katalogu skryptu jako bazy dla ścieżek plików
script_dir = os.path.dirname(os.path.abspath(__file__))

# Funkcja do wczytywania konfiguracji
def load_config():
    filename = os.path.join(script_dir, 'config.json')
    with open(filename, 'r') as file:
        return json.load(file)

# Funkcja do wczytywania listy stacji radiowych
def load_radio_stations():
    filename = os.path.join(script_dir, 'stations.json')
    with open(filename, 'r') as file:
        return json.load(file)

# Wczytywanie konfiguracji i listy stacji
config = load_config()
radio_stations = load_radio_stations()

# Przypisanie konfiguracji do zmiennych
ftp_config = config['ftp_config']
webdav_config = config['webdav_config']
nextcloud_folder_path = config['nextcloud_folder_path']
audio_config = config['audio_config']

def ensure_nextcloud_folder_path(client, base_path, remote_path_suffix):
    """
    Zapewnia, że cała ścieżka folderów istnieje na Nextcloud.
    """
    folders = remote_path_suffix.strip('/').split('/')
    current_path = base_path.rstrip('/')  # Usuń ewentualne końcowe slash'e z base_path

    for folder in folders[:-1]:  # Pomin ostatni element, który jest nazwą pliku
        current_path = f"{current_path}/{folder}"  # Zbuduj ścieżkę krok po kroku
        try:
            if not client.check(current_path):  # Jeśli folder nie istnieje, check() zwróci False
                client.mkdir(current_path)
                print(f"Utworzono folder na Nextcloud: {current_path}")
        except Exception as e:
            print(f"Wystąpił błąd podczas sprawdzania/tworzenia folderu {current_path} na Nextcloud: {e}")
            break  # Przerwij pętlę, jeśli wystąpi błąd

def manage_ftp_space(folder_path, manage_space):
    if not manage_space:
        return
    with FTP(ftp_config['ftp_hostname']) as ftp:
        ftp.login(ftp_config['ftp_username'], ftp_config['ftp_password'])
        ftp.cwd(os.path.join(ftp_config['ftp_base_path'], folder_path))

        files = ftp.nlst()
        audio_files = [file for file in files if file.endswith('.mp3')]

        while len(audio_files) > ftp_config['max_files']:
            oldest_file = min(audio_files, key=lambda x: ftp.sendcmd('MDTM ' + x))
            ftp.delete(oldest_file)
            print(f"Usunięto najstarszy plik: {oldest_file}")
            audio_files.remove(oldest_file)

def upload_to_ftp(full_local_path, remote_path_suffix, manage_space):
    with FTP(ftp_config['ftp_hostname']) as ftp:
        ftp.login(ftp_config['ftp_username'], ftp_config['ftp_password'])
        ftp.cwd(ftp_config['ftp_base_path'])
        dirs = remote_path_suffix.strip('/').split('/')
        for dir in dirs[:-1]:
            try:
                ftp.cwd(dir)
            except error_perm:
                ftp.mkd(dir)
                ftp.cwd(dir)
        with open(full_local_path, 'rb') as file:
            ftp.storbinary('STOR ' + dirs[-1], file)
        print(f"Plik {remote_path_suffix} został przesłany na serwer FTP.")
    manage_ftp_space('/'.join(dirs[:-1]), manage_space)

def upload_to_nextcloud(full_local_path, remote_path_suffix):
    client = Client(webdav_config)
    ensure_nextcloud_folder_path(client, nextcloud_folder_path, remote_path_suffix)  # Zapewnienie istnienia ścieżki folderów
    remote_path = os.path.join(nextcloud_folder_path, remote_path_suffix)
    try:
        client.upload_sync(remote_path=remote_path, local_path=full_local_path)
        print(f"Plik {remote_path_suffix} został przesłany na Nextcloud.")
    except Exception as e:
        print(f"Wystąpił błąd podczas przesyłania pliku do Nextcloud: {e}")

def record_stream(station_name, duration, program_name, storage_option, keep_local, manage_space):
    stream_url = radio_stations.get(station_name)
    if not stream_url:
        print(f"Stacja {station_name} nie jest skonfigurowana.")
        return

    bitrate = audio_config['bitrate']
    channels = '2' if audio_config['channels'] == 'stereo' else '1'

    current_time = datetime.now().strftime("%Y-%m-%d_%H_%M")
    filename = f"{station_name}_{program_name}_{current_time}.mp3"
    local_save_path = os.path.join(script_dir, 'recordings', station_name, program_name)
    if not os.path.exists(local_save_path):
        os.makedirs(local_save_path)
    full_local_path = os.path.join(local_save_path, filename)

    command = [
        'ffmpeg',
        '-t', str(duration * 60),
        '-i', stream_url,
        '-acodec', 'libmp3lame',
        '-b:a', bitrate,
        '-ac', channels,
        '-vn',
        full_local_path
    ]

    try:
        subprocess.run(command, check=True)
        print(f"Nagranie zapisane lokalnie jako: {full_local_path}")

        if 'FTP' in storage_option:
            upload_to_ftp(full_local_path, os.path.join(station_name, program_name, filename), manage_space)
        if 'NC' in storage_option:
            upload_to_nextcloud(full_local_path, os.path.join(station_name, program_name, filename))
        if keep_local == 'temp':
            os.remove(full_local_path)
            print(f"Plik {filename} został usunięty z dysku lokalnego.")
    except subprocess.CalledProcessError as e:
        print(f"Wystąpił błąd podczas nagrywania: {e}")

if __name__ == '__main__':
    if len(sys.argv) < 7:
        print("Użycie: radiosend.py <NazwaStacji> <CzasNagrywania> <NazwaProgramu> <FTP/NC/FTP+NC/none> <save/temp> [savespaceFTP/dontsavespaceFTP]")
    else:
        station_name = sys.argv[1].lower()
        duration = int(sys.argv[2])
        program_name = sys.argv[3]
        storage_option = sys.argv[4].upper()
        keep_local = sys.argv[5].lower()
        manage_space = 'savespaceFTP' in sys.argv and 'dontsavespaceFTP' not in sys.argv
        record_stream(station_name, duration, program_name, storage_option, keep_local, manage_space)
